// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.helpers;

import java.awt.*;

import javax.swing.JPanel;

public class LayoutManagerHandler
{
  public static enum Direction
  {
    horizontal, vertical
  }

  public static enum Resize
  {
    horizontal, vertical, both
  }

  private final Direction direction;
  private final GridBagLayout gridbag;
  private final GridBagConstraints constraints;
  private final JPanel panel;

  public LayoutManagerHandler(Direction direction)
  {
    this.direction = direction;
    gridbag = new GridBagLayout();
    constraints = new GridBagConstraints();
    constraints.insets = new Insets(2, 2, 2, 2);
    if (direction == Direction.horizontal) {
      constraints.weighty = 1.0;
      constraints.gridheight = GridBagConstraints.REMAINDER;
    } else {
      constraints.weightx = 1.0;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
    }
    panel = new JPanel(gridbag);
  }

  public void addComponent(Component component, boolean resize, double weight)
  {
    if (resize) {
      constraints.fill = GridBagConstraints.BOTH;
    } else {
      switch (direction) {
      case horizontal:
        constraints.fill = GridBagConstraints.VERTICAL;
        break;
      case vertical:
        constraints.fill = GridBagConstraints.HORIZONTAL;
        break;
      }
    }

    if (direction == Direction.horizontal) {
      constraints.weightx = weight;
    } else {
      constraints.weighty = weight;
    }

    gridbag.setConstraints(component, constraints);
    panel.add(component);
  }

  public JPanel getPanel()
  {
    return panel;
  }
}
